@@ -1,21 +1,18 @@
 import asyncio
 import logging
+import platform
 import queue
 import threading
 import time
 
-from bleak import BleakClient, discover
+import bleak
 
-from pylgbst.comms import Connection, MOVE_HUB_HW_UUID_CHAR
+from pylgbst.comms import Connection, MOVE_HUB_HW_UUID_CHAR, MOVE_HUB_HW_UUID_SERV
 
 log = logging.getLogger('comms-bleak')
 
-# Queues to handle request / responses. Acts as a buffer between API and async BLE driver
-resp_queue = queue.Queue()
-req_queue = queue.Queue()
 
-
-class BleakDriver(object):
+class BleakDriver(Connection):
     """Driver that provides interface between API and Bleak."""
 
     def __init__(self, hub_mac=None, hub_name=None):
@@ -31,6 +28,10 @@ class BleakDriver(object):
         self._connection_thread = None
         self._processing_thread = None
 
+        # Queues to handle request / responses. Acts as a buffer between API and async BLE driver
+        self.resp_queue = queue.Queue()
+        self.req_queue = queue.Queue()
+
     def set_notify_handler(self, handler):
         """
         Set handler function used to communicate with an API.
@@ -57,27 +58,31 @@ class BleakDriver(object):
 
     async def _bleak_thread(self):
         bleak = BleakConnection()
-        await bleak.connect(self.hub_mac, self.hub_name)
-        await bleak.set_notify_handler(self._safe_handler)
+        # For MacOS 12+ the service_uuids kwarg is required for scanning
+        kwargs = {}
+        if "Darwin" == platform.system() and int(platform.mac_ver()[0].split(".")[0]) >= 12:
+            kwargs = {"service_uuids": [MOVE_HUB_HW_UUID_SERV]}
+        await bleak.connect(self.hub_mac, self.hub_name, **kwargs)
+        await bleak.set_notify_handler((self._safe_handler, self.resp_queue))
         # After connecting, need to send any data or hub will drop the connection,
         # below command is Advertising name request update
         await bleak.write_char(MOVE_HUB_HW_UUID_CHAR, bytearray([0x05, 0x00, 0x01, 0x01, 0x05]))
         while not self._abort:
             await asyncio.sleep(0.1)
-            if req_queue.qsize() != 0:
-                data = req_queue.get()
+            if self.req_queue.qsize() != 0:
+                data = self.req_queue.get()
                 await bleak.write(data[0], data[1])
 
         logging.info("Communications thread has exited")
 
     @staticmethod
-    def _safe_handler(handler, data):
+    def _safe_handler(handler, data, resp_queue):
         resp_queue.put((handler, data))
 
     def _processing(self):
         while not self._abort:
-            if resp_queue.qsize() != 0:
-                msg = resp_queue.get()
+            if self.resp_queue.qsize() != 0:
+                msg = self.resp_queue.get()
                 self._handler(msg[0], bytes(msg[1]))
 
             time.sleep(0.01)
@@ -95,7 +100,7 @@ class BleakDriver(object):
         if not self._connection_thread.is_alive() or not self._processing_thread.is_alive():
             raise ConnectionError('Something went wrong, communication threads not functioning.')
 
-        req_queue.put((handle, data))
+        self.req_queue.put((handle, data))
 
     def disconnect(self):
         """
@@ -129,17 +134,20 @@ class BleakConnection(Connection):
         logging.getLogger('bleak.backends.dotnet.client').setLevel(logging.WARNING)
         logging.getLogger('bleak.backends.bluezdbus.client').setLevel(logging.WARNING)
 
-    async def connect(self, hub_mac=None, hub_name=None):
+    async def connect(self, hub_mac=None, hub_name=None, **kwargs):
         """
         Connect to device.
 
-        :param hub_mac: Optional Lego HUB MAC to connect to.
+        :param hub_mac: Optional Lego HUB MAC to connect to
+        :param hub_name: Optional Lego Hub name to connect to
+        :kwargs: Optional parameters for bleak.discover
+
         :raises ConnectionError: When cannot connect to given MAC or name matching fails.
         :return: None
         """
         log.info("Discovering devices... Press green button on Hub")
         for i in range(0, 30):
-            devices = await discover(timeout=1)
+            devices = await bleak.discover(timeout=1, **kwargs)
             log.debug("Devices: %s", devices)
             for dev in devices:
                 log.debug(dev)
@@ -156,7 +164,7 @@ class BleakConnection(Connection):
         else:
             raise ConnectionError('Device not found.')
 
-        self._client = BleakClient(self._device.address)
+        self._client = bleak.BleakClient(self._device)
         status = await self._client.connect()
         log.debug('Connection status: {status}'.format(status=status))
 
@@ -191,17 +199,18 @@ class BleakConnection(Connection):
         """
         await self._client.write_gatt_char(characteristic_uuid, data)
 
-    async def set_notify_handler(self, handler):
+    async def set_notify_handler(self, inputs):
         """
         Set notification handler.
 
         :param handler: Handle function to be called when receive any data.
         :return: None
         """
+        handler, resp_queue = inputs
 
         def c(handle, data):
             log.debug('Response: {handle} {payload}'.format(handle=handle, payload=[hex(x) for x in data]))
-            handler(handle, data)
+            handler(handle, data, resp_queue)
 
         await self._client.start_notify(MOVE_HUB_HW_UUID_CHAR, c)
 
