@@ -1,5 +1,4 @@
 import threading
-import time
 
 from pylgbst import get_connection_auto
 from pylgbst.messages import *
@@ -7,19 +6,20 @@ from pylgbst.peripherals import *
 from pylgbst.utilities import queue
 from pylgbst.utilities import str2hex, usbyte, ushort
 
-log = logging.getLogger('hub')
+log = logging.getLogger("hub")
 
 PERIPHERAL_TYPES = {
     DevTypes.MOTOR: Motor,
+    DevTypes.SYSTEM_TRAIN_MOTOR: TrainMotor,
     DevTypes.MOTOR_EXTERNAL_TACHO: EncodedMotor,
     DevTypes.MOTOR_INTERNAL_TACHO: EncodedMotor,
     DevTypes.VISION_SENSOR: VisionSensor,
     DevTypes.RGB_LIGHT: LEDRGB,
+    DevTypes.LED_LIGHT: LEDLight,
     DevTypes.TILT_EXTERNAL: TiltSensor,
     DevTypes.TILT_INTERNAL: TiltSensor,
     DevTypes.CURRENT: Current,
     DevTypes.VOLTAGE: Voltage,
-
     DevTypes.DUPLO_TRAIN_BASE_MOTOR.value: Motor,
     # DUPLO_TRAIN_BASE_SPEAKER = 42
     DevTypes.DUPLO_TRAIN_BASE_COLOR_SENSOR: VisionSensor,
@@ -34,7 +34,7 @@ PERIPHERAL_TYPES = {
     # DevTypes.TECHNIC_MEDIUM_HUB_ACCELEROMETER: 57
     # DevTypes.TECHNIC_MEDIUM_HUB_GYRO_SENSOR: 58
     DevTypes.TECHNIC_MEDIUM_HUB_TILT_SENSOR: TiltSensor,
-    # DevTypes.TECHNIC_MEDIUM_HUB_TEMPERATURE_SENSOR: 60
+    DevTypes.TECHNIC_MEDIUM_HUB_TEMPERATURE_SENSOR: Temperature,
     DevTypes.TECHNIC_COLOR_SENSOR: VisionSensor,
     DevTypes.TECHNIC_DISTANCE_SENSOR: VisionSensor,
     # DevTypes.TECHNIC_FORCE_SENSOR: 63  # Spike Prime
@@ -46,11 +46,12 @@ PERIPHERAL_TYPES = {
 }
 
 
-class Hub(object):
+class Hub:
     """
     :type connection: pylgbst.comms.Connection
     :type peripherals: dict[int,Peripheral]
     """
+
     HUB_HARDWARE_HANDLE = 0x0E
 
     def __init__(self, connection=None):
@@ -61,6 +62,7 @@ class Hub(object):
         self._sync_lock = threading.Lock()
 
         self.add_message_handler(MsgHubAttachedIO, self._handle_device_change)
+        self.add_message_handler(MsgPortOutputFeedback, self._handle_output_feedback)
         self.add_message_handler(MsgPortValueSingle, self._handle_sensor_data)
         self.add_message_handler(MsgPortValueCombined, self._handle_sensor_data)
         self.add_message_handler(MsgGenericError, self._handle_error)
@@ -150,8 +152,11 @@ class Hub(object):
 
     def _handle_device_change(self, msg):
         if msg.event == MsgHubAttachedIO.EVENT_DETACHED:
-            log.debug("Detaching peripheral: %s", self.peripherals[msg.port])
-            self.peripherals.pop(msg.port)
+            if msg.port not in self.peripherals:
+                log.warning("Strange: got detach command for port %s that is not attached, will ignore it", msg.port)
+            else:
+                log.info("Detaching peripheral: %s", self.peripherals[msg.port])
+                self.peripherals.pop(msg.port)
             return
 
         assert msg.event in (msg.EVENT_ATTACHED, msg.EVENT_ATTACHED_VIRTUAL)
@@ -167,7 +172,7 @@ class Hub(object):
 
             self.peripherals[port] = Peripheral(self, port)
 
-        log.info("Attached peripheral: %s", self.peripherals[msg.port])
+        log.info("Attached peripheral %s => %s", DevTypes(dev_type).name, self.peripherals[msg.port])
 
         if msg.event == msg.EVENT_ATTACHED:
             hw_revision = reversed([usbyte(msg.payload, x) for x in range(2, 6)])
@@ -177,6 +182,20 @@ class Hub(object):
         elif msg.event == msg.EVENT_ATTACHED_VIRTUAL:
             self.peripherals[port].virtual_ports = (usbyte(msg.payload, 2), usbyte(msg.payload, 3))
 
+    def _handle_output_feedback(self, msg):
+        assert isinstance(msg, MsgPortOutputFeedback)
+        if msg.port not in self.peripherals:
+            log.warning("Notification on port with no device: %s", msg.port)
+            return
+
+        device = self.peripherals[msg.port]
+        if hasattr(device, "cmd_in_progress"):
+            device.cmd_in_progress = msg.is_in_progress()
+            if device.cmd_in_progress:
+                log.debug("Command on device %s in progress.", device)
+            else:
+                log.debug("Command on device %s completed.", device)
+
     def _handle_sensor_data(self, msg):
         assert isinstance(msg, (MsgPortValueSingle, MsgPortValueCombined))
         if msg.port not in self.peripherals:
@@ -203,8 +222,8 @@ class MoveHub(Hub):
     :type current: Current
     :type voltage: Voltage
     :type vision_sensor: pylgbst.peripherals.VisionSensor
-    :type port_C: Peripheral
-    :type port_D: Peripheral
+    :type port_C: pylgbst.peripherals.Peripheral
+    :type port_D: pylgbst.peripherals.Peripheral
     :type motor_A: EncodedMotor
     :type motor_B: EncodedMotor
     :type motor_AB: EncodedMotor
@@ -230,7 +249,7 @@ class MoveHub(Hub):
         if connection is None:
             connection = get_connection_auto(hub_name=self.DEFAULT_NAME)
 
-        super(MoveHub, self).__init__(connection)
+        super().__init__(connection)
         self.info = {}
 
         # shorthand fields
@@ -281,8 +300,11 @@ class MoveHub(Hub):
     # noinspection PyTypeChecker
     def _handle_device_change(self, msg):
         with self._comm_lock:
-            super(MoveHub, self)._handle_device_change(msg)
-            if isinstance(msg, MsgHubAttachedIO) and msg.event != MsgHubAttachedIO.EVENT_DETACHED:
+            super()._handle_device_change(msg)
+            if (
+                    isinstance(msg, MsgHubAttachedIO)
+                    and msg.event != MsgHubAttachedIO.EVENT_DETACHED
+            ):
                 port = msg.port
                 if port == self.PORT_A:
                     self.motor_A = self.peripherals[port]
@@ -305,14 +327,18 @@ class MoveHub(Hub):
 
                 if type(self.peripherals[port]) == VisionSensor:
                     self.vision_sensor = self.peripherals[port]
-                elif type(self.peripherals[port]) == EncodedMotor \
-                        and port not in (self.PORT_A, self.PORT_B, self.PORT_AB):
+                elif type(self.peripherals[port]) == EncodedMotor and port not in (
+                        self.PORT_A,
+                        self.PORT_B,
+                        self.PORT_AB,
+                ):
                     self.motor_external = self.peripherals[port]
 
 
 class SmartHub(Hub):
     """
     Class implementing Lego SmartHub specifics
+    https://www.lego.com/en-pt/product/hub-88009
 
     :type led: LEDRGB
     :type current: Current
@@ -321,7 +347,7 @@ class SmartHub(Hub):
     :type port_B: Peripheral
     """
 
-    DEFAULT_NAME = 'Smart Hub'
+    DEFAULT_NAME = "Smart Hub"
 
     # PORTS
     PORT_A = 0x00
@@ -334,7 +360,7 @@ class SmartHub(Hub):
         if connection is None:
             connection = get_connection_auto(hub_name=self.DEFAULT_NAME)
 
-        super(SmartHub, self).__init__(connection)
+        super().__init__(connection)
 
         self.button = Button(self)
         self.led = None
@@ -348,6 +374,7 @@ class SmartHub(Hub):
     def _wait_for_devices(self, get_dev_set=None):
         if not get_dev_set:
             get_dev_set = lambda: (self.led, self.current, self.voltage)
+
         for num in range(0, 100):
             devices = get_dev_set()
             if all(devices):
@@ -359,8 +386,11 @@ class SmartHub(Hub):
 
     # noinspection PyTypeChecker
     def _handle_device_change(self, msg):
-        super(SmartHub, self)._handle_device_change(msg)
-        if isinstance(msg, MsgHubAttachedIO) and msg.event != MsgHubAttachedIO.EVENT_DETACHED:
+        super()._handle_device_change(msg)
+        if (
+                isinstance(msg, MsgHubAttachedIO)
+                and msg.event != MsgHubAttachedIO.EVENT_DETACHED
+        ):
             port = msg.port
             if port == self.PORT_A:
                 self.port_A = self.peripherals[port]
