@@ -1,14 +1,22 @@
 import logging
-import math
+import time
 import traceback
 from struct import pack, unpack
 from threading import Thread
 
-from pylgbst.messages import MsgHubProperties, MsgPortOutput, MsgPortInputFmtSetupSingle, MsgPortInfoRequest, \
-    MsgPortModeInfoRequest, MsgPortInfo, MsgPortModeInfo, MsgPortInputFmtSingle
-from pylgbst.utilities import queue, str2hex, usbyte, ushort, usint
+from pylgbst.messages import (
+    MsgHubProperties,
+    MsgPortOutput,
+    MsgPortInputFmtSetupSingle,
+    MsgPortInfoRequest,
+    MsgPortModeInfoRequest,
+    MsgPortInfo,
+    MsgPortModeInfo,
+    MsgPortInputFmtSingle,
+)
+from pylgbst.utilities import queue, str2hex, usbyte, ushort, usint, abs_scaled_100
 
-log = logging.getLogger('peripherals')
+log = logging.getLogger("peripherals")
 
 # COLORS
 COLOR_BLACK = 0x00
@@ -21,7 +29,7 @@ COLOR_GREEN = 0x06
 COLOR_YELLOW = 0x07
 COLOR_ORANGE = 0x08
 COLOR_RED = 0x09
-COLOR_WHITE = 0x0a
+COLOR_WHITE = 0x0A
 COLOR_NONE = 0xFF
 COLORS = {
     COLOR_BLACK: "BLACK",
@@ -35,14 +43,15 @@ COLORS = {
     COLOR_ORANGE: "ORANGE",
     COLOR_RED: "RED",
     COLOR_WHITE: "WHITE",
-    COLOR_NONE: "NONE"
+    COLOR_NONE: "NONE",
 }
 
 
 # TODO: support more types of peripherals from
 # https://lego.github.io/lego-ble-wireless-protocol-docs/index.html#io-type-id
 
-class Peripheral(object):
+
+class Peripheral:
     """
     :type parent: pylgbst.hub.Hub
     :type _incoming_port_data: queue.Queue
@@ -54,7 +63,6 @@ class Peripheral(object):
         :type parent: pylgbst.hub.Hub
         :type port: int
         """
-        super(Peripheral, self).__init__()
         self.virtual_ports = ()
         self.hub = parent
         self.port = port
@@ -66,8 +74,8 @@ class Peripheral(object):
 
         self._incoming_port_data = queue.Queue(1)  # limit 1 means we drop data if we can't handle it fast enough
         thr = Thread(target=self._queue_reader)
-        thr.setDaemon(True)
-        thr.setName("Port data queue: %s" % self)
+        thr.daemon = True
+        thr.name = "Port data queue: %s" % self
         thr.start()
 
     def __repr__(self):
@@ -87,9 +95,11 @@ class Peripheral(object):
             update_delta = self._port_mode.upd_delta
             log.debug("Implied update delta=%s", update_delta)
 
-        if self._port_mode.mode == mode \
-                and self._port_mode.upd_enabled == send_updates \
-                and self._port_mode.upd_delta == update_delta:
+        if (
+                self._port_mode.mode == mode
+                and self._port_mode.upd_enabled == send_updates
+                and self._port_mode.upd_delta == update_delta
+        ):
             log.debug("Already in target mode, no need to switch")
             return
         else:
@@ -111,7 +121,10 @@ class Peripheral(object):
 
     def subscribe(self, callback, mode=0x00, granularity=1):
         if self._port_mode.mode != mode and self._subscribers:
-            raise ValueError("Port is in active mode %r, unsubscribe all subscribers first" % self._port_mode)
+            raise ValueError(
+                "Port is in active mode %r, unsubscribe all subscribers first"
+                % self._port_mode
+            )
         self.set_port_mode(mode, True, granularity)
         if callback:
             self._subscribers.add(callback)
@@ -137,7 +150,7 @@ class Peripheral(object):
             log.debug("Dropped port data: %r", msg)
 
     def _decode_port_data(self, msg):
-        """
+        """Return the sensor value according to the current sensor mode
         :rtype: tuple
         """
         log.warning("Unhandled port data: %r", msg)
@@ -172,23 +185,25 @@ class Peripheral(object):
                 "synchronizable": mode_info.is_synchronizable(),
                 "can_output": mode_info.is_output(),
                 "can_input": mode_info.is_input(),
-            }
+            },
         }
 
         if mode_info.is_combinable():
-            mode_combinations = self.hub.send(MsgPortInfoRequest(self.port, MsgPortInfoRequest.INFO_MODE_COMBINATIONS))
+            mode_combinations = self.hub.send(
+                MsgPortInfoRequest(self.port, MsgPortInfoRequest.INFO_MODE_COMBINATIONS)
+            )
             assert isinstance(mode_combinations, MsgPortInfo)
             info['possible_mode_combinations'] = mode_combinations.possible_mode_combinations
 
-        info['modes'] = []
+        info["modes"] = []
         for mode in range(256):
-            info['modes'].append(self._describe_mode(mode))
+            info["modes"].append(self._describe_mode(mode))
 
         for mode in mode_info.output_modes:
-            info['output_modes'].append(self._describe_mode(mode))
+            info["output_modes"].append(self._describe_mode(mode))
 
         for mode in mode_info.input_modes:
-            info['input_modes'].append(self._describe_mode(mode))
+            info["input_modes"].append(self._describe_mode(mode))
 
         log.debug("Port info for 0x%x: %s", self.port, info)
         return info
@@ -212,13 +227,29 @@ class LEDRGB(Peripheral):
     MODE_RGB = 0x01
 
     def __init__(self, parent, port):
-        super(LEDRGB, self).__init__(parent, port)
+        super().__init__(parent, port)
 
     def set_color(self, color):
+        """Set color of the RGB LED
+
+        :param color: Accept 2 types of data:
+            - RGB: tuple of 3 values,
+            - index: 1 value choosen among 12 color values:
+            `COLOR_BLACK, COLOR_PINK, COLOR_PURPLE, COLOR_BLUE, COLOR_LIGHTBLUE,
+            COLOR_CYAN, COLOR_GREEN, COLOR_YELLOW, COLOR_ORANGE, COLOR_RED,
+            COLOR_WHITE, COLOR_NONE`.
+            Note that `COLOR_BLACK` and `COLOR_NONE` turn LED off.
+        :type color: <tuple> or <int> or constant
+        """
         if isinstance(color, (list, tuple)):
             assert len(color) == 3, "RGB color has to have 3 values"
             self.set_port_mode(self.MODE_RGB)
-            payload = pack("<B", self.MODE_RGB) + pack("<B", color[0]) + pack("<B", color[1]) + pack("<B", color[2])
+            payload = (
+                    pack("<B", self.MODE_RGB)
+                    + pack("<B", color[0])
+                    + pack("<B", color[1])
+                    + pack("<B", color[2])
+            )
         else:
             if color == COLOR_NONE:
                 color = COLOR_BLACK
@@ -233,13 +264,102 @@ class LEDRGB(Peripheral):
         self._send_output(msg)
 
     def _decode_port_data(self, msg):
+        """Decode data emitted by the hub
+
+        .. note:: Experimental function for now; Most of the time, hardware
+            shouldn't return any data. Also note that its seems not possible
+            to retrieve the current RGB color on the peripheral.
+            See issue #111.
+
+        :param msg: Message retrieved from the hub.
+        :return: Tuple of length dependent of the current mode (1 for MODE_INDEX,
+            3 for MODE_RGB)
+        """
         if len(msg.payload) == 3:
-            return usbyte(msg.payload, 0), usbyte(msg.payload, 1), usbyte(msg.payload, 2),
+            return (
+                usbyte(msg.payload, 0),
+                usbyte(msg.payload, 1),
+                usbyte(msg.payload, 2),
+            )
         else:
             return usbyte(msg.payload, 0),
 
+    # Set color of the RGB LED (no getter)
+    color = property(fset=set_color)
 
-class Motor(Peripheral):
+
+class LEDLight(Peripheral):
+    """Support of headlight kit (LPF2-LIGHT)"""
+
+    MODE_BRIGHTNESS = 0x00
+
+    def __init__(self, parent, port):
+        super().__init__(parent, port)
+
+    def set_brightness(self, brightness):
+        """Set brightness of LEDs
+
+        :param brightness: Number between 0 and 100%.
+        :type brightness: <int> or <float>
+        """
+        if (
+                not isinstance(brightness, (int, float))
+                or brightness > 100
+                or brightness < 0
+        ):
+            raise ValueError("Brightness must be a number between 0 and 100")
+
+        self.set_port_mode(self.MODE_BRIGHTNESS)
+        payload = pack("<B", self.MODE_BRIGHTNESS) + pack("<B", int(brightness))
+
+        msg = MsgPortOutput(self.port, MsgPortOutput.WRITE_DIRECT_MODE_DATA, payload)
+        self._send_output(msg)
+
+    @property
+    def brightness(self):
+        """Get current brightness value
+        :rtype: <int>
+        """
+        return self.get_sensor_data(LEDLight.MODE_BRIGHTNESS)[0]
+
+    @brightness.setter
+    def brightness(self, value):
+        """Set brightness of LEDs
+        .. seealso:: `set_brightness`
+        """
+        self.set_brightness(value)
+
+    def _decode_port_data(self, msg):
+        return usbyte(msg.payload, 0),
+
+
+class BaseMotor(Peripheral):
+    def _write_direct_mode(self, subcmd, params):
+        params = pack("<B", subcmd) + params
+        msg = MsgPortOutput(self.port, MsgPortOutput.WRITE_DIRECT_MODE_DATA, params)
+        self._send_output(msg)
+
+
+class TrainMotor(BaseMotor):
+    """
+    Simple DC motor (Lego 88011).
+    See https://github.com/undera/pylgbst/issues/129
+    """
+    SUBCMD_POWER = 0x00
+    SUBCMD_1 = 0x01  # TODO: figure out what it does. We know it's not sensor mode.
+
+    def power(self, param=1.0):
+        """
+        Power the motor, with value -1.0..1.0
+        """
+        params = pack("<b", abs_scaled_100(param))
+        self._write_direct_mode(self.SUBCMD_POWER, params)
+
+    def stop(self):
+        self.power(0)
+
+
+class Motor(BaseMotor):
     SUBCMD_START_POWER = 0x01
     SUBCMD_START_POWER_GROUPED = 0x02
     SUBCMD_SET_ACC_TIME = 0x05
@@ -253,33 +373,23 @@ class Motor(Peripheral):
     END_STATE_HOLD = 126
     END_STATE_FLOAT = 0
 
-    def _speed_abs(self, relative):
+    def __init__(self, parent, port):
+        super().__init__(parent, port)
+        self.cmd_in_progress = False
+
+    def _speed_abs(self, relative):  # FIXME: it's not "speed", rather it's a "power"
         if relative == Motor.END_STATE_BRAKE or relative == Motor.END_STATE_HOLD:
             # special value for BRAKE
             # https://lego.github.io/lego-ble-wireless-protocol-docs/index.html#output-sub-command-startpower-power
             return relative
 
-        if relative < -1:
-            log.warning("Speed cannot be less than -1")
-            relative = -1
+        return abs_scaled_100(relative)
 
-        if relative > 1:
-            log.warning("Speed cannot be more than 1")
-            relative = 1
-
-        absolute = math.ceil(relative * 100)  # scale of 100 is proven by experiments
-        return int(absolute)
-
-    def _write_direct_mode(self, subcmd, params):
-        params = pack("<B", subcmd) + params
-        msg = MsgPortOutput(self.port, MsgPortOutput.WRITE_DIRECT_MODE_DATA, params)
-        self._send_output(msg)
-
-    def _send_cmd(self, subcmd, params):
+    def _send_cmd(self, subcmd, params, wait_complete=True):
         if self.virtual_ports:
             subcmd += 1  # de-facto rule
 
-        msg = MsgPortOutput(self.port, subcmd, params)
+        msg = MsgPortOutput(self.port, subcmd, params, wait_complete)
         self._send_output(msg)
 
     def start_power(self, power_primary=1.0, power_secondary=None):
@@ -342,7 +452,7 @@ class Motor(Peripheral):
         self._send_cmd(self.SUBCMD_START_SPEED, params)
 
     def timed(self, seconds, speed_primary=1.0, speed_secondary=None, max_power=1.0, end_state=END_STATE_BRAKE,
-              use_profile=0b11):
+              use_profile=0b11, wait_complete=True):
         """
         https://lego.github.io/lego-ble-wireless-protocol-docs/index.html#output-sub-command-startspeedfortime-time-speed-maxpower-endstate-useprofile-0x09
         """
@@ -359,7 +469,11 @@ class Motor(Peripheral):
         params += pack("<B", end_state)
         params += pack("<B", use_profile)
 
-        self._send_cmd(self.SUBCMD_START_SPEED_FOR_TIME, params)
+        self._send_cmd(self.SUBCMD_START_SPEED_FOR_TIME, params, wait_complete)
+
+    def wait_complete(self):
+        while self.cmd_in_progress:
+            time.sleep(0.01)
 
 
 class EncodedMotor(Motor):
@@ -375,7 +489,7 @@ class EncodedMotor(Motor):
     SENSOR_TEST = 0x03  # exists, but neither input nor output mode
 
     def angled(self, degrees, speed_primary=1.0, speed_secondary=None, max_power=1.0, end_state=Motor.END_STATE_BRAKE,
-               use_profile=0b11):
+               use_profile=0b11, wait_complete=True):
         """
         https://lego.github.io/lego-ble-wireless-protocol-docs/index.html#output-sub-command-startspeedfordegrees-degrees-speed-maxpower-endstate-useprofile-0x0b
         :type degrees: int
@@ -400,10 +514,10 @@ class EncodedMotor(Motor):
         params += pack("<B", end_state)
         params += pack("<B", use_profile)
 
-        self._send_cmd(self.SUBCMD_START_SPEED_FOR_DEGREES, params)
+        self._send_cmd(self.SUBCMD_START_SPEED_FOR_DEGREES, params, wait_complete)
 
     def goto_position(self, degrees_primary, degrees_secondary=None, speed=1.0, max_power=1.0,
-                      end_state=Motor.END_STATE_BRAKE, use_profile=0b11):
+                      end_state=Motor.END_STATE_BRAKE, use_profile=0b11, wait_complete=True):
         """
         https://lego.github.io/lego-ble-wireless-protocol-docs/index.html#output-sub-command-gotoabsoluteposition-abspos-speed-maxpower-endstate-useprofile-0x0d
         """
@@ -421,22 +535,22 @@ class EncodedMotor(Motor):
         params += pack("<B", end_state)
         params += pack("<B", use_profile)
 
-        self._send_cmd(self.SUBCMD_GOTO_ABSOLUTE_POSITION, params)
+        self._send_cmd(self.SUBCMD_GOTO_ABSOLUTE_POSITION, params, wait_complete)
 
     def _decode_port_data(self, msg):
         data = msg.payload
         if self._port_mode.mode == self.SENSOR_ANGLE:
             angle = unpack("<l", data[0:4])[0]
-            return (angle,)
+            return angle,
         elif self._port_mode.mode == self.SENSOR_SPEED:
             speed = unpack("<b", data[0:1])[0]
-            return (speed,)
+            return speed,
         else:
             log.debug("Got motor sensor data while in unexpected mode: %r", self._port_mode)
             return ()
 
     def subscribe(self, callback, mode=SENSOR_ANGLE, granularity=1):
-        super(EncodedMotor, self).subscribe(callback, mode, granularity)
+        super().subscribe(callback, mode, granularity)
 
     def preset_encoder(self, degrees=0, degrees_secondary=None, only_combined=False):
         """
@@ -494,36 +608,36 @@ class TiltSensor(Peripheral):
     }
 
     def subscribe(self, callback, mode=MODE_3AXIS_SIMPLE, granularity=1):
-        super(TiltSensor, self).subscribe(callback, mode, granularity)
+        super().subscribe(callback, mode, granularity)
 
     def _decode_port_data(self, msg):
         data = msg.payload
         if self._port_mode.mode == self.MODE_2AXIS_ANGLE:
-            roll = unpack('<b', data[0:1])[0]
-            pitch = unpack('<b', data[1:2])[0]
-            return (roll, pitch)
+            roll = unpack("<b", data[0:1])[0]
+            pitch = unpack("<b", data[1:2])[0]
+            return roll, pitch
         elif self._port_mode.mode == self.MODE_3AXIS_SIMPLE:
             state = usbyte(data, 0)
-            return (state,)
+            return state,
         elif self._port_mode.mode == self.MODE_2AXIS_SIMPLE:
             state = usbyte(data, 0)
-            return (state,)
+            return state,
         elif self._port_mode.mode == self.MODE_IMPACT_COUNT:
             bump_count = usint(data, 0)
-            return (bump_count,)
+            return bump_count,
         elif self._port_mode.mode == self.MODE_3AXIS_ACCEL:
-            roll = unpack('<b', data[0:1])[0]
-            pitch = unpack('<b', data[1:2])[0]
-            yaw = unpack('<b', data[2:3])[0]  # did I get the order right?
-            return (roll, pitch, yaw)
+            roll = unpack("<b", data[0:1])[0]
+            pitch = unpack("<b", data[1:2])[0]
+            yaw = unpack("<b", data[2:3])[0]  # did I get the order right?
+            return roll, pitch, yaw
         elif self._port_mode.mode == self.MODE_ORIENT_CF:
             state = usbyte(data, 0)
-            return (state,)
+            return state,
         elif self._port_mode.mode == self.MODE_IMPACT_CF:
             state = usbyte(data, 0)
-            return (state,)
+            return state,
         elif self._port_mode.mode == self.MODE_CALIBRATION:
-            return (usbyte(data, 0), usbyte(data, 1), usbyte(data, 2))
+            return usbyte(data, 0), usbyte(data, 1), usbyte(data, 2)
         else:
             log.debug("Got tilt sensor data while in unexpected mode: %r", self._port_mode)
             return ()
@@ -541,50 +655,50 @@ class VisionSensor(Peripheral):
     SET_COLOR = 0x05
     COLOR_RGB = 0x06
     SET_IR_TX = 0x07
-    COLOR_DISTANCE_FLOAT = 0x08  # it's not declared by dev's mode info
+    COLOR_DISTANCE_FLOAT = 0x08
 
     DEBUG = 0x09  # first val is by fact ambient light, second is zero
-    CALIBRATE = 0x0a  # gives constant values
+    CALIBRATE = 0x0A  # gives constant values
 
     def __init__(self, parent, port):
-        super(VisionSensor, self).__init__(parent, port)
+        super().__init__(parent, port)
 
     def subscribe(self, callback, mode=COLOR_DISTANCE_FLOAT, granularity=1):
-        super(VisionSensor, self).subscribe(callback, mode, granularity)
+        super().subscribe(callback, mode, granularity)
 
     def _decode_port_data(self, msg):
         data = msg.payload
         if self._port_mode.mode == self.COLOR_INDEX:
             color = usbyte(data, 0)
-            return (color,)
+            return color,
         elif self._port_mode.mode == self.COLOR_DISTANCE_FLOAT:
             color = usbyte(data, 0)
             val = usbyte(data, 1)
             partial = usbyte(data, 3)
             if partial:
                 val += 1.0 / partial
-            return (color, float(val))
+            return color, float(val)
         elif self._port_mode.mode == self.DISTANCE_INCHES:
             val = usbyte(data, 0)
-            return (val,)
+            return val,
         elif self._port_mode.mode == self.DISTANCE_REFLECTED:
             val = usbyte(data, 0) / 100.0
-            return (val,)
+            return val,
         elif self._port_mode.mode == self.AMBIENT_LIGHT:
             val = usbyte(data, 0) / 100.0
-            return (val,)
+            return val,
         elif self._port_mode.mode == self.COUNT_2INCH:
             count = usint(data, 0)
-            return (count,)
+            return count,
         elif self._port_mode.mode == self.COLOR_RGB:
             val1 = int(255 * ushort(data, 0) / 1023.0)
             val2 = int(255 * ushort(data, 2) / 1023.0)
             val3 = int(255 * ushort(data, 4) / 1023.0)
-            return (val1, val2, val3)
+            return val1, val2, val3
         elif self._port_mode.mode == self.DEBUG:
             val1 = 10 * ushort(data, 0) / 1023.0
             val2 = 10 * ushort(data, 2) / 1023.0
-            return (val1, val2)
+            return val1, val2
         elif self._port_mode.mode == self.CALIBRATE:
             return [ushort(data, x * 2) for x in range(8)]
         else:
@@ -592,6 +706,11 @@ class VisionSensor(Peripheral):
             return ()
 
     def set_color(self, color):
+        """Set color of the RGB LED on the sensor
+
+        :param color:
+            Note that `COLOR_BLACK` and `COLOR_NONE` turn LED off.
+        """
         if color == COLOR_NONE:
             color = COLOR_BLACK
 
@@ -612,33 +731,103 @@ class VisionSensor(Peripheral):
         msg = MsgPortOutput(self.port, MsgPortOutput.WRITE_DIRECT_MODE_DATA, payload)
         self._send_output(msg)
 
+    @property
+    def color(self):
+        """Get index of the current color
+        :return: See values of `COLORS` module variable.
+        :rtype: <int>
+        """
+        return self.get_sensor_data(self.COLOR_INDEX)[0]
+
+    @property
+    def distance(self):
+        """Get measured distance of an object to the sensor
+        :return: Value from 0 to 10
+        :rtype: <int>
+        """
+        return self.get_sensor_data(self.DISTANCE_INCHES)[0]
+
+    @property
+    def reflected_light(self):
+        """Get measured reflected light
+        :return: Value from 0 to 1.0
+        :rtype: <float>
+        """
+        return self.get_sensor_data(self.DISTANCE_REFLECTED)[0]
+
+    @property
+    def luminosity(self):
+        """Get the current luminosity in lux
+        :return: Value from 0 to 1.0
+        :rtype: <float>
+        """
+        return self.get_sensor_data(self.AMBIENT_LIGHT)[0]
+
+    @property
+    def detection_count(self):
+        """Get the number of object detections ~2 inches in front of sensor
+        :rtype: <int>
+        """
+        return self.get_sensor_data(self.COUNT_2INCH)[0]
+
+    @property
+    def rgb_color(self):
+        """Get detected RGB channels
+
+        :return: Tuple of 3 values for RGB channels
+        :rtype: <tuple <int>, <int>, <int>>
+        """
+        return self.get_sensor_data(self.COLOR_RGB)
+
 
 class Voltage(Peripheral):
+    """Retrieve voltage information from the hub"""
+
     # sensor says there are "L" and "S" values, but what are they?
     VOLTAGE_L = 0x00
     VOLTAGE_S = 0x01
 
     def __init__(self, parent, port):
-        super(Voltage, self).__init__(parent, port)
+        super().__init__(parent, port)
 
     def _decode_port_data(self, msg):
         data = msg.payload
         val = ushort(data, 0)
         volts = 9600.0 * val / 3893.0 / 1000.0
-        return (volts,)
+        return volts,
+
+    @property
+    def voltage(self):
+        """Get the measured voltage of the battery
+
+        :return: Return the value (in Volts) of the VOLTAGE_L mode by default.
+        :rtype: <float>
+        """
+        return self.get_sensor_data(self.VOLTAGE_L)[0]
 
 
 class Current(Peripheral):
+    """Retrieve current information from the hub"""
+
     CURRENT_L = 0x00
     CURRENT_S = 0x01
 
     def __init__(self, parent, port):
-        super(Current, self).__init__(parent, port)
+        super().__init__(parent, port)
 
     def _decode_port_data(self, msg):
         val = ushort(msg.payload, 0)
         milliampers = 2444 * val / 4095.0
-        return (milliampers,)
+        return milliampers,
+
+    @property
+    def current(self):
+        """Get the measured current of the battery
+
+        :return: Return the value (in mA) of the CURRENT_L mode by default.
+        :rtype: <float>
+        """
+        return self.get_sensor_data(self.CURRENT_L)[0]
 
 
 class Button(Peripheral):
@@ -647,7 +836,7 @@ class Button(Peripheral):
     """
 
     def __init__(self, parent):
-        super(Button, self).__init__(parent, 0)  # fake port 0
+        super().__init__(parent, 0)  # fake port 0
         self.hub.add_message_handler(MsgHubProperties, self._props_msg)
 
     def subscribe(self, callback, mode=None, granularity=1):
@@ -667,5 +856,29 @@ class Button(Peripheral):
         """
         :type msg: MsgHubProperties
         """
-        if msg.property == MsgHubProperties.BUTTON and msg.operation == MsgHubProperties.UPSTREAM_UPDATE:
+        if (
+                msg.property == MsgHubProperties.BUTTON
+                and msg.operation == MsgHubProperties.UPSTREAM_UPDATE
+        ):
             self._notify_subscribers(usbyte(msg.parameters, 0))
+
+
+class Temperature(Peripheral):
+    """Get battery temperature from the hub"""
+    MODE_TEMP = 0
+
+    def __init__(self, parent, port):
+        super().__init__(parent, port)
+
+    def _decode_port_data(self, msg):
+        # Fix temp with a small offset to get the real temperature
+        magic_offset = 2.1
+        return (unpack("<h", msg.payload)[0] / 10) - magic_offset,
+
+    @property
+    def temperature(self):
+        """Get the measured temperature in C°
+
+        :rtype: <float>
+        """
+        return self.get_sensor_data(self.MODE_TEMP)[0]
