@@ -4,6 +4,7 @@ This module offers some utilities, in a way they are work in both Python 2 and 3
 
 import binascii
 import logging
+import math
 import sys
 from struct import unpack
 
@@ -43,3 +44,16 @@ def str2hex(data):  # we need it for python 2+3 compatibility
         data = bytes(data, "ascii")
     hexed = binascii.hexlify(data)
     return hexed
+
+
+def abs_scaled_100(relative):
+    if relative < -1.0:
+        log.warning("Speed cannot be less than -1")
+        relative = -1.0
+
+    if relative > 1.0:
+        log.warning("Speed cannot be more than 1")
+        relative = 1.0
+
+    absolute = math.ceil(relative * 100)  # scale of 100 is proven by experiments
+    return int(absolute)
